import java.awt.Color; // Color for colorizing
import java.util.LinkedList;

/**
 * DeerAgent -- Deer as a subclass of Agent
 **/
public class DeerAgent extends Agent {
  
  /** class constant for the color */
  private static final Color brown = new Color(116,64,35);
  
  /** class constant for how far deer can smell */
  private static final double SMELL_RANGE = 50;
  
  /** Collection of all Deer */
  public static LinkedList allDeer = new LinkedList();
  
  /**
   * Initialize, by adding to Deer list
   **/
  public void init(Simulation thisSim){
    // Do the normal initializations
    super.init(thisSim);
    
    // Make it brown
    setColor(brown);
    
    // Add to list of Deer
    allDeer.add(this);
  }
  
  /**
   * To die, do normal stuff, but
   * also remove from deer list
   **/
  public void die(){
    super.die();
    allDeer.remove(this);
    System.out.println("Deer left: "+allDeer.size());
  }
  
  /**
   * How a DeerAgent acts
   **/
  public void act()
  {
    // get the closest wolf within the smell range
    WolfAgent closeWolf = (WolfAgent) getClosest(SMELL_RANGE,
                                WolfAgent.allWolves);
    
    if (closeWolf != null) {
      // Turn to face the wolf
      this.turnToFace(closeWolf);
      // Now directly in the opposite direction
      this.turn(180);
      // How far to run? How about half of current speed??
      this.forward((int) (speed/2));
    }
    else {
      // Run the normal act() -- wander aimlessly
      super.act();
    }
  }
  
    ////////////////////////////// Constructors ////////////////////////
  // Copy this section AS-IS into subclasses, but rename Agent to 
  // Your class.
  
  /**
   * Constructor that takes the model display (the original
   * position will be randomly assigned)
   * @param modelDisplayer thing that displays the model
   * @param thisSim my simulation
   */
  public DeerAgent (ModelDisplay modelDisplayer,Simulation thisSim)
  {
    super(randNumGen.nextInt(modelDisplayer.getWidth()),
          randNumGen.nextInt(modelDisplayer.getHeight()),
          modelDisplayer, thisSim);
  }
  
  /** Constructor that takes the x and y and a model
   * display to draw it on
   * @param x the starting x position
   * @param y the starting y position
   * @param modelDisplayer the thing that displays the model
   * @param thisSim my simulation
   */
  public DeerAgent (int x, int y, ModelDisplay modelDisplayer, 
               Simulation thisSim) 
  {
    // let the parent constructor handle it
    super(x,y,modelDisplayer,thisSim);
  }
  

}
