import jm.music.data.*;
import jm.JMC;
import jm.util.*;
import jm.music.tools.*;
 
public class SongElement {
  // Every element knows its next element and its part (of the score)
  private SongElement next;
  private Part myPart;
  
  // When we make a new element, the next part is empty, and ours is a blank new part
  public SongElement(){
    this.next = null;
    this.myPart = new Part();
  }
  
  // setPhrase takes a phrase and makes it the one for this element 
  // at the desired start time with the given instrument
  public void setPhrase(Phrase myPhrase, double startTime, int instrument){
     myPhrase.setStartTime(startTime);
    this.myPart.addPhrase(myPhrase);
    this.myPart.setInstrument(instrument);
  }
  
  
  // Here are the two methods needed to make a linked list of elements
  public void setNext(SongElement nextOne){
    this.next = nextOne;
  }
  
  public SongElement next(){
    return this.next;
  }
  
  // We could just access myPart directly
  // but we can CONTROL access by using a method
  // (called an accessor)
  private Part part(){
    return this.myPart;
  }
  
  // Why do we need this?
  // If we want one piece to start after another, we need
  // to know when the last one ends.
  // Notice: It's the phrase that knows the end time.
  //    We have to ask the part for its phrase (assuming only one)
  //    to get the end time.
  public double getEndTime(){
    return this.myPart.getPhrase(0).getEndTime();
  }
  
  // We need setChannel because each part has to be in its
  // own channel if it has different start times.
  // So, we'll set the channel when we assemble the score.
  private void setChannel(int channel){
    myPart.setChannel(channel);
  }
  
  public void showFromMeOn(){
    // Make the score that we'll assemble the elements into
    // We'll set it up with a default time signature and tempo we like
    // (Should probably make it possible to change these -- maybe with inputs?)
     Score myScore = new Score("My Song");
     myScore.setTimeSignature(3,4);
     myScore.setTempo(120.0);
     
     // Each element will be in its own channel
     int channelCount = 1;
     
     // Start from this element (this)
     SongElement current = this;
     // While we're not through...
     while (current != null)
     {
       // Set the channel, increment the channel, then add it in.
       current.setChannel(channelCount);
       channelCount = channelCount + 1;
       myScore.addPart(current.part());
       
       // Now, move on to the next element
       // which we already know isn't null
       current = current.next();
     };
     
     // At the end, let's see it!
     View.notate(myScore);

  }
  
}
